@@ -1,6 +1,6 @@
 from ..base.tools import resolveRandom, randomProbabilities, checkProbabilities
 from math import log
-from ..base.Model import Model
+from ..base.Base_MC import *
 from ..base.Set import Set
 from .Scheduler import Scheduler
 from numpy.random import geometric
@@ -9,11 +9,11 @@ from ast import literal_eval
 from multiprocessing import cpu_count, Pool
 from random import choices
 
-class MDP(Model):
+class MDP(Base_MC):
 	"""
 	Class representing a MDP.
 	"""
-	def __init__(self,matrix: ndarray, labeling: list, actions:list, name: str="unknown_MDP"):
+	def __init__(self,matrix: ndarray, labelling: list, actions:list, name: str="unknown_MDP"):
 		"""
 		Create a MDP.
 
@@ -24,9 +24,9 @@ class MDP(Model):
 			`matrix[s1][act_ID][s2][obs_ID]` is the probability of moving 
 			from `s1` to `s2` by executing action of ID `act_ID` and seeing 
 			the observation of ID `obs_ID`.
-		labeling: list of str
+		labelling: list of str
 			A list of N observations (with N the nb of states).
-			If `labeling[s] == o` then state of ID `s` is labelled by `o`.
+			If `labelling[s] == o` then state of ID `s` is labelled by `o`.
 			Each state has exactly one label.
 		actions: list of str
 			The list of all possible actions, such that:
@@ -34,20 +34,10 @@ class MDP(Model):
 		name : str, optional
 			Name of the model. Default is "unknow_MDP"
 		"""
+		self.model_type = MDP_ID
 		self.actions = actions
 		self.nb_actions = len(self.actions)
-		self.labeling = labeling
-		self.alphabet = list(set(labeling))
-
-		if not 'init' in self.labeling:
-			msg = "No initial state given: at least one"
-			msg += " state should be labelled by 'init'."
-			raise ValueError(msg)
-		initial_state = [1.0/self.labeling.count("init") if i=='init' else 0.0 for i in self.labeling]
-
-		super().__init__(matrix,initial_state,name)
-		if len(labeling) != self.nb_states:
-			raise ValueError("The length of labeling is not equal to the number of states")
+		super().__init__(matrix,labelling,name)
 		for i in range(self.nb_states):
 			for a in range(self.nb_actions):
 				if not checkProbabilities(matrix[i][a]):
@@ -83,44 +73,6 @@ class MDP(Model):
 		else:
 			return list(set(self.actions[i] for i in where(self.matrix[state].sum(axis=1) > 0.0)[0]))
 
-	def getAlphabet(self) -> list:
-		"""
-		Returns the alphabet of this model.
-
-		Returns
-		-------
-		list of str
-			The alphabet of this model
-		
-		Example
-		-------
-		>>> model.getAlphabet()
-		['a','b','c','d','done']
-		"""
-		return self.alphabet
-	
-	def getLabel(self,state: int) -> str:
-		"""
-		Returns the label of `state`.
-
-		Parameters
-		----------
-		state : int
-			a state ID
-
-		Returns
-		-------
-		str
-			a label
-
-		Example
-		-------
-		>>> model.getLabel(2)
-		'Label-of-state-2'
-		"""
-		self._checkStateIndex(state)
-		return self.labeling[state]
-
 
 	def tau(self,s1: int,action: str,s2: int,obs: str) -> float:
 		"""
@@ -158,7 +110,7 @@ class MDP(Model):
 		"""
 		self._checkStateIndex(s1)
 		self._checkStateIndex(s2)
-		if obs != self.labeling[s1]:
+		if obs != self.labelling[s1]:
 			return 0.0
 		if action not in self.actions:
 			return 0.0
@@ -191,7 +143,6 @@ class MDP(Model):
 		self._checkStateIndex(s2)
 		return self.matrix[s1][self.actions.index(action)][s2]
 	
-
 	def next(self,state: int, action: str) -> tuple:
 		"""
 		Return a state-observation pair according to the distributions 
@@ -229,9 +180,9 @@ class MDP(Model):
 		1.0
 		"""
 		c = resolveRandom(self.matrix[state][self.actions.index(action)])
-		return (c, self.labeling[state])
+		return (c, self.labelling[state])
 			
-	def run(self,number_steps: int,scheduler: Scheduler) -> list:
+	def run(self,number_steps: int,scheduler: Scheduler, current: int = -1) -> list:
 		"""
 		Simulates a run of length ``number_steps`` of the model under
 		``scheduler`` and returns the sequence of actions-observations generated.
@@ -240,6 +191,9 @@ class MDP(Model):
 		----------
 		number_steps: int
 			length of the simulation.
+		current : int, optional.
+			If current it set, it starts from the state `current`.
+			Otherwise it starts from an initial state.
 
 		Returns
 		-------
@@ -247,7 +201,8 @@ class MDP(Model):
 			List of alterning state-observation.
 		"""
 		res = []
-		current = resolveRandom(self.initial_state)
+		if current == -1:
+			current = resolveRandom(self.initial_state)
 		scheduler.reset()
 		current_len = 0
 		while current_len < number_steps:
@@ -262,7 +217,7 @@ class MDP(Model):
 			scheduler.addObservation(observation)
 			current = next_state
 			current_len += 1
-		res.append(self.labeling[current])
+		res.append(self.labelling[current])
 		return res
 
 	def generateSet(self, set_size: int, param, scheduler: Scheduler, distribution=None, min_size=None) -> list:
@@ -315,7 +270,7 @@ class MDP(Model):
 
 		return Set(seq,val,t=1)
 	
-	def save(self,file_path:str):
+	def save(self,file_path:str) -> None:
 		"""Save the model into a text file.
 
 		Parameters
@@ -329,21 +284,41 @@ class MDP(Model):
 		"""
 		f = open(file_path, 'w')
 		f.write("MDP\n")
-		f.write(str(self.labeling))
-		f.write('\n')
 		f.write(str(self.actions))
 		f.write('\n')
 		super()._save(f)
 	
 	def _stateToString(self,state:int) -> str:
-		res = "----STATE "+str(state)+"--"+self.labeling[state]+"----\n"
+		res = "----STATE "+str(state)+"--"+self.labelling[state]+"----\n"
 		for ai,a in enumerate(self.actions):
 			for s in range(self.nb_states):
 				if self.matrix[state][ai][s] > 0.0001:
 					res += "s"+str(state)+" - ("+a+") -> s"+str(s)+" : "+str(self.matrix[state][ai][s])+'\n'
 		return res
 	
+	def savePrism(self,file_path:str) -> None:
+		"""
+		Save this model into `file_path` in the Prism format.
 
+		Parameters
+		----------
+		file_path : str
+			Path of the output file.
+		"""
+		f = open(file_path,'w')
+		f.write("mdp\n\n")
+		super()._savePrism(f)
+	
+	def _savePrismMatrix(self,f) -> None:
+		for s1 in range(self.nb_states):
+			for ai,a in enumerate(self.actions):
+				if (self.matrix[s1,ai]!=0).any():
+					res = '\t['+a+'] s='+str(s1)+' ->'
+					for s2 in where(self.matrix[s1,ai] != 0.0)[0]:
+						res += ' '+str(self.matrix[s1,ai,s2])+":(s'="+str(s2)+") +"
+					res = res[:-2]+';\n'
+					f.write(res)
+		
 	def _logLikelihood_oneproc(self,sequences: Set) -> float:
 		"""
 		Compute the average loglikelihood of a set of sequences.
@@ -376,7 +351,7 @@ class MDP(Model):
 			common = int(common/2)
 			alpha_matrix = self._updateAlphaMatrix(sequence_obs,sequence_actions,common,alpha_matrix)
 			
-			last_arr = alpha_matrix[-1] * (array(self.labeling) == sequence[-1])
+			last_arr = alpha_matrix[-1] * (array(self.labelling) == sequence[-1])
 			last_arr = last_arr.sum()
 			if last_arr > 0.0:
 				loglikelihood += log(last_arr) * times
@@ -464,7 +439,7 @@ class MDP(Model):
 				p = array([self.tau(ss,sequence[k+1],s,sequence[k]) for ss in range(self.nb_states)])
 				new_arr[s] = dot(prev_arr,p)
 			prev_arr = new_arr
-		prev_arr = prev_arr*(array(self.labeling) == sequence[-1])
+		prev_arr = prev_arr*(array(self.labelling) == sequence[-1])
 		if prev_arr.sum() == 0.0:
 			return 0.0
 		return log(prev_arr.sum())*times
@@ -488,17 +463,17 @@ def loadMDP(file_path: str) -> MDP:
 	if l != "MDP":
 		msg = " this file doesn't describe an MC: it describes a "+l
 		raise ValueError(msg)
-	labeling = literal_eval(f.readline()[:-1])
 	actions = literal_eval(f.readline()[:-1])
+	labelling = literal_eval(f.readline()[:-1])
 	name = f.readline()[:-1]
 	initial_state = array(literal_eval(f.readline()[:-1]))
 	matrix = literal_eval(f.readline()[:-1])
 	matrix = array(matrix)
 	f.close()
-	return MDP(matrix, labeling, actions, name)
+	return MDP(matrix, labelling, actions, name)
 
 
-def MDP_random(nb_states: int,alphabet: list, actions: list,random_initial_state: bool = True, deterministic: bool = False) -> MDP:
+def MDP_random(nb_states: int,labelling: list, actions: list,random_initial_state: bool = True, deterministic: bool = False) -> MDP:
 	"""
 	Generate a random MDP.
 
@@ -506,7 +481,7 @@ def MDP_random(nb_states: int,alphabet: list, actions: list,random_initial_state
 	----------
 	number_states : int
 		Number of states.
-	alphabet : list of str
+	labelling : list of str
 		List of observations.
 	actions : list of str
 		List of actions.	
@@ -514,7 +489,8 @@ def MDP_random(nb_states: int,alphabet: list, actions: list,random_initial_state
 		If set to True we will start in each state with a random probability, otherwise we will always start in state 0.
 		Default is True.
 	deterministic: bool, optional
-		If True, the model will be determinstic: in state `s`, with action `a`, there is only one transition labelled with `o`.
+		If True, the model will be determinstic: in state `s`, with action `a`, for all label `o`,
+		there is at most one transition to a state labelled with `o`.
 		Default is False.
 	
 	Returns
@@ -522,17 +498,7 @@ def MDP_random(nb_states: int,alphabet: list, actions: list,random_initial_state
 	MDP
 		A pseudo-randomly generated MDP.
 	"""
-	if 'init' in alphabet:
-		msg =  "The label 'init' cannot be used: it is reserved for initial states."
-		raise SyntaxError(msg)
-
-	if nb_states < len(alphabet):
-		print("WARNING: the size of the alphabet is geater than the",end=" ")
-		print("number of states. Some labels will not be assigned to",end=" ")
-		print("any states.")
-	labeling = alphabet[:min(len(alphabet),nb_states)] + choices(alphabet,k=nb_states-len(alphabet))
-	alphabet = list(set(labeling))
-
+	alphabet = list(set(labelling))
 	matrix = []
 	for s in range(nb_states):
 		if not deterministic:
@@ -542,7 +508,7 @@ def MDP_random(nb_states: int,alphabet: list, actions: list,random_initial_state
 		else:
 			matrix.append([])
 			for a in actions:
-				dest = [choices(where(array(labeling) == o)[0]) for o in alphabet]
+				dest = [choices(where(array(labelling) == o)[0]) for o in alphabet]
 				p = zeros(nb_states)
 				probs = randomProbabilities(len(alphabet))
 				for i,j in zip(dest,probs):
@@ -555,11 +521,12 @@ def MDP_random(nb_states: int,alphabet: list, actions: list,random_initial_state
 		init = [1.0]+[0.0 for i in range(nb_states)]
 	matrix.append(array([init for a in actions]))
 	matrix = array(matrix)
-	labeling.append('init')
-	return MDP(matrix, labeling, actions, "MDP_random_"+str(nb_states)+"_states")
+	
+	labelling = labelsForRandomModel(nb_states,labelling)
+	return MDP(matrix, labelling, actions, "MDP_random_"+str(nb_states)+"_states")
 
 
-def createMDP(transitions:list, labeling:list, initial_state, name: str ="unknown_MDP") -> MDP:
+def createMDP(transitions:list, labelling:list, initial_state, name: str ="unknown_MDP") -> MDP:
 	"""
 	An user-friendly way to create an MDP.
 
@@ -568,9 +535,9 @@ def createMDP(transitions:list, labeling:list, initial_state, name: str ="unknow
 	transitions : [ list of tuples (int, str, int, float)]
 		Each tuple represents a transition as follow: 
 		(source state ID, action, destination state ID, probability).
-	labeling: list of str
+	labelling: list of str
 		A list of N observations (with N the nb of states).
-		If `labeling[s] == o` then state of ID `s` is labelled by `o`.
+		If `labelling[s] == o` then state of ID `s` is labelled by `o`.
 		Each state has exactly one label.
 	initial_state : int or list of float
 		Determine which state is the initial one (then it's the id of the
@@ -583,12 +550,12 @@ def createMDP(transitions:list, labeling:list, initial_state, name: str ="unknow
 	Returns
 	-------
 	MDP
-		the MDP describes by `transitions`, `labeling`, and `initial_state`.
+		the MDP describes by `transitions`, `labelling`, and `initial_state`.
 	
 	Examples
 	--------
 	"""
-	if 'init' in labeling:
+	if 'init' in labelling:
 		msg =  "The label 'init' cannot be used: it is reserved for initial states."
 		raise SyntaxError(msg)
 
@@ -599,16 +566,16 @@ def createMDP(transitions:list, labeling:list, initial_state, name: str ="unknow
 	actions.sort()
 	nb_actions = len(actions)
 	
-	if nb_states > len(labeling):
-		raise ValueError("all states are not labelled (the labeling list is too small).")
-	elif nb_states < len(labeling):
-		print("WARNING: the labeling list is bigger than the number of states")
+	if nb_states > len(labelling):
+		raise ValueError("all states are not labelled (the labelling list is too small).")
+	elif nb_states < len(labelling):
+		print("WARNING: the labelling list is bigger than the number of states")
 
 	res = zeros((nb_states,nb_actions,nb_states))
 	for t in transitions:
 		res[states.index(t[0])][actions.index(t[1])][states.index(t[2])] = t[3]
 	
-	labeling.append('init')
+	labelling.append('init')
 	res = vstack((res,zeros((1,nb_actions,nb_states))))
 	res = concatenate((res,zeros((nb_states+1,nb_actions,1))),axis=2)
 	if type(initial_state) == int:
@@ -620,4 +587,4 @@ def createMDP(transitions:list, labeling:list, initial_state, name: str ="unknow
 		for a in range(nb_actions):
 			res[-1][a] = array(initial_state+[0.0])
 	
-	return MDP(res,labeling,actions,name)
\ No newline at end of file
+	return MDP(res,labelling,actions,name)
\ No newline at end of file
